@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2007, 2008, 2009, 2010 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999, 2001, 2007-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "imap4d.h"
 
@@ -32,12 +29,16 @@ imap4d_uid (struct imap4d_command *command, imap4d_tokbuf_t tok)
   char *err_text = "Completed";
 
   if (imap4d_tokbuf_argc (tok) < 3)
-    return util_finish (command, RESP_BAD, "Invalid arguments");
+    return io_completion_response (command, RESP_BAD, "Invalid arguments");
 
   cmd = imap4d_tokbuf_getarg (tok, IMAP4_ARG_1);
   
   if (mu_c_strcasecmp (cmd, "FETCH") == 0)
-    rc = imap4d_fetch0 (tok, 1, &err_text);
+    {
+      int xlev = set_xscript_level (MU_XSCRIPT_PAYLOAD);
+      rc = imap4d_fetch0 (tok, 1, &err_text);
+      set_xscript_level (xlev);
+    }
   else if (mu_c_strcasecmp (cmd, "COPY") == 0)
     rc = imap4d_copy0 (tok, 1, &err_text);
   else if (mu_c_strcasecmp (cmd, "STORE") == 0)
@@ -49,5 +50,5 @@ imap4d_uid (struct imap4d_command *command, imap4d_tokbuf_t tok)
       err_text = "Uknown uid command";
       rc = RESP_BAD;
     }
-  return util_finish (command, rc, "%s %s", cmd, err_text);
+  return io_completion_response (command, rc, "%s %s", cmd, err_text);
 }
