@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2009-2012 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -22,11 +22,14 @@
 extern "C" {
 #endif
 
+# include <mailutils/types.h>
+  
 int mu_strlower (char *);
 int mu_strupper (char *);
 
 int mu_c_strcasecmp (const char *a, const char *b);
 int mu_c_strncasecmp (const char *a, const char *b, size_t n);
+char *mu_c_strcasestr (const char *haystack, const char *needle);
 
 size_t mu_rtrim_class (char *str, int __class);
 size_t mu_rtrim_cset (char *str, const char *cset);
@@ -40,6 +43,8 @@ char *mu_str_skip_class_comp (const char *str, int __class);
 char *mu_str_skip_cset_comp (const char *str, const char *cset);
 
 char *mu_str_stripws (char *string);  
+
+int mu_string_split (const char *string, char *delim, mu_list_t list);
   
 #ifdef __cplusplus
 }
