@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2009, 2010 Free Software
+   Copyright (C) 1999-2001, 2005, 2007, 2009-2012 Free Software
    Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -13,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifndef _MAILUTILS_FOLDER_H
 # define _MAILUTILS_FOLDER_H
@@ -32,6 +31,7 @@ struct mu_list_response
   int level;
   int separator;
   char *name;
+  mu_record_t format;   /* Associated mailbox format record */
 };
 
 typedef int (*mu_folder_match_fp) (const char *, void *, int);
@@ -62,22 +62,26 @@ extern int  mu_folder_lsub           (mu_folder_t, const char *, const char *,
 				      mu_list_t *);
 
 /* Stream settings.  */
-extern int  mu_folder_get_stream     (mu_folder_t, mu_stream_t *);
+extern int  mu_folder_get_stream     (mu_folder_t, mu_stream_t *)
+                                       __attribute__ ((deprecated));
+extern int  mu_folder_get_streamref  (mu_folder_t, mu_stream_t *);
+  
 extern int  mu_folder_set_stream     (mu_folder_t, mu_stream_t);
 
   /* Match function */
 extern int mu_folder_set_match (mu_folder_t folder, mu_folder_match_fp pmatch);
 extern int mu_folder_get_match (mu_folder_t folder,
 				mu_folder_match_fp *pmatch);
+
+  /* Two often used matchers: */
+  /* 1. The default: IMAP-style wildcards: */
+extern int mu_folder_imap_match (const char *name, void *pattern, int flags);
+  /* 2. UNIX-style glob(7) wildcards: */
+extern int mu_folder_glob_match (const char *name, void *pattern, int flags);
   
   /* Notifications.  */
 extern int  mu_folder_get_observable (mu_folder_t, mu_observable_t *);
 
-  /* Debug.  */
-extern int  mu_folder_has_debug      (mu_folder_t folder);
-extern int  mu_folder_get_debug      (mu_folder_t, mu_debug_t *);
-extern int  mu_folder_set_debug      (mu_folder_t, mu_debug_t);
-
 /* Authentication.  */
 extern int  mu_folder_get_authority  (mu_folder_t, mu_authority_t *);
 extern int  mu_folder_set_authority  (mu_folder_t, mu_authority_t);
@@ -86,10 +90,16 @@ extern int  mu_folder_set_authority  (mu_folder_t, mu_authority_t);
 extern int  mu_folder_get_url        (mu_folder_t, mu_url_t *);
 extern int  mu_folder_set_url        (mu_folder_t, mu_url_t);
 
+/* Property */
+extern int  mu_folder_set_property   (mu_folder_t, mu_property_t);  
+extern int  mu_folder_get_property   (mu_folder_t, mu_property_t *);
+  
 /* FIXME: not implemented */
 extern int  mu_folder_decrement      (mu_folder_t);
 
 extern void mu_list_response_free    (void *data);
+
+int _mu_fsfolder_init (mu_folder_t folder);
   
 #ifdef __cplusplus
 }
