@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2004, 2005, 2007, 2010 Free Software
+   Copyright (C) 1999-2000, 2004-2005, 2007, 2010-2012 Free Software
    Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -13,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see 
+   <http://www.gnu.org/licenses/>. */
 
 #ifndef _MAILUTILS_ITERATOR_H
 #define _MAILUTILS_ITERATOR_H
@@ -73,6 +72,8 @@ extern int mu_iterator_set_itrctl (mu_iterator_t itr,
 				   int (*itrctl) (void *,
 						  enum mu_itrctl_req,
 						  void *));
+extern int mu_iterator_set_dataptr (mu_iterator_t itr,
+				    void *(*dataptr) (void *));
 #ifdef __cplusplus
 }
 #endif
