@@ -1,6 +1,6 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2006, 2007, 2009 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2004, 2006-2007, 2009-2012 Free Software Foundation,
+   Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,10 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA
-*/
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #include <mailutils/cpp/list.h>
 
@@ -203,7 +201,7 @@ List :: size ()
 void
 List :: apply (mu_list_action_t* action, void* cbdata)
 {
-  int status = mu_list_do (mu_list, action, cbdata);
+  int status = mu_list_foreach (mu_list, action, cbdata);
   if (status)
     throw Exception ("List::apply", status);
 }
@@ -214,12 +212,10 @@ List :: set_comparator (mu_list_comparator_t comp)
   return mu_list_set_comparator (mu_list, comp);
 }
 
-void
-List :: set_destroy_item (void (*mu_destroy_item) (void *item))
+mu_list_destroy_item_t
+List :: set_destroy_item (mu_list_destroy_item_t mu_destroy_item)
 {
-  int status = mu_list_set_destroy_item (mu_list, mu_destroy_item);
-  if (status)
-    throw Exception ("List::set_destroy_item", status);
+  return mu_list_set_destroy_item (mu_list, mu_destroy_item);
 }
 
 std::list<void*>
