@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2008, 2010 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2002, 2005, 2007-2008, 2010-2012 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -25,6 +24,16 @@
 
 mu_list_t mu_sieve_include_path = NULL;
 mu_list_t mu_sieve_library_path = NULL;
+mu_list_t mu_sieve_library_path_prefix = NULL;
+
+mu_debug_handle_t mu_sieve_debug_handle;
+
+void
+mu_sieve_debug_init ()
+{
+  if (!mu_sieve_debug_handle)
+    mu_sieve_debug_handle = mu_debug_register_category ("sieve");
+}
 
 static int
 _path_append (void *item, void *data)
@@ -53,12 +62,18 @@ mu_sieve_module_init (enum mu_gocs_op op, void *data)
 
   if (p->clearflags & MU_SIEVE_CLEAR_INCLUDE_PATH)
     mu_list_destroy (&mu_sieve_include_path);
-  mu_list_do (p->include_path, _path_append, &mu_sieve_include_path);
+  mu_list_foreach (p->include_path, _path_append, &mu_sieve_include_path);
   if (p->clearflags & MU_SIEVE_CLEAR_LIBRARY_PATH)
-    mu_list_destroy (&mu_sieve_library_path);
-  mu_list_do (p->library_path, _path_append, &mu_sieve_library_path);
-  mu_sv_load_add_path (mu_sieve_library_path);
+    {
+      mu_list_destroy (&mu_sieve_library_path);
+      mu_list_destroy (&mu_sieve_library_path_prefix);
+    }
+  mu_list_foreach (p->library_path_prefix, _path_append,
+	      &mu_sieve_library_path_prefix);
+  mu_list_foreach (p->library_path, _path_append, &mu_sieve_library_path);
   mu_list_destroy (&p->library_path);
+  mu_list_destroy (&p->library_path_prefix);
   mu_list_destroy (&p->include_path);
+  mu_sieve_debug_init ();
   return 0;
 }
