@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 2010-2012 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -31,13 +31,6 @@ mu_imap_destroy (mu_imap_t *pimap)
     {
       mu_imap_t imap = *pimap;
       
-      /* Free the response buffer.  */
-      if (imap->tagbuf)
-	free (imap->tagbuf);
-      /* Free the read buffer.  */
-      if (imap->rdbuf)
-	free (imap->rdbuf);
-
       if (imap->errstr)
 	free (imap->errstr);
 
@@ -46,10 +39,9 @@ mu_imap_destroy (mu_imap_t *pimap)
       if (imap->tag_buf)
 	free (imap->tag_buf);
       
-      mu_list_destroy (&imap->untagged_resp);
       mu_list_destroy (&imap->capa);
       
-      mu_stream_destroy (&imap->carrier);
+      mu_imapio_destroy (&imap->io);
 
       free (imap);
 
