@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008,
-   2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 1999-2001, 2003-2012 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -29,6 +28,7 @@
 #include <mailutils/io.h>
 #include <mailutils/filter.h>
 #include <mailutils/cctype.h>
+#include <mailutils/datetime.h>
 
 #define ATTRIBUTE_IS_DELETED(flag)        (flag & MU_ATTRIBUTE_DELETED)
 #define ATTRIBUTE_IS_EQUAL(flag1, flag2)  (flag1 == flag2)
@@ -590,7 +590,7 @@ mbox_get_message (mu_mailbox_t mailbox, size_t msgno, mu_message_t *pmsg)
   /* Sanity checks.  */
   if (pmsg == NULL)
     return MU_ERR_OUT_PTR_NULL;
-  if (mud == NULL)
+  if (mud == NULL || msgno < 1)
     return EINVAL;
 
   /* If we did not start a scanning yet do it now.  */
@@ -605,7 +605,7 @@ mbox_get_message (mu_mailbox_t mailbox, size_t msgno, mu_message_t *pmsg)
   if (!(mud->messages_count > 0
 	&& msgno > 0
 	&& msgno <= mud->messages_count))
-    return EINVAL;
+    return MU_ERR_NOENT;
 
   mum = mud->umessages[msgno - 1];
 
@@ -859,22 +859,23 @@ restore_date (mu_message_t msg, char **pret)
   mu_header_t hdr;
   const char *s;
   char *date = NULL;
-  time_t t;
+  struct tm tm;
+  struct mu_timezone tz;
   
   if (mu_message_get_header (msg, &hdr) == 0)
     mu_header_sget_value (hdr, MU_HEADER_DATE, &s);
 
-  if (s && mu_parse_date (s, &t, NULL))
+  if (s && mu_scan_datetime (s, MU_DATETIME_SCAN_RFC822, &tm, &tz, NULL) == 0)
     {
-      char datebuf[MU_ENVELOPE_DATE_LENGTH+1];
+      char datebuf[MU_DATETIME_FROM_LENGTH+1];
       
-      /* FIXME: 1. Preserve TZ info */
-      mu_strftime (datebuf, sizeof datebuf, MU_ENVELOPE_DATE_FORMAT,
-		   localtime (&t));
+      /* FIXME: Compensate for TZ differences. */
+      mu_strftime (datebuf, sizeof datebuf, MU_DATETIME_FROM, &tm);
       date = strdup (datebuf);
     }
   else
     {
+      time_t t;
       time (&t);
       date = strdup (ctime (&t));
     }
