@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2007, 2009, 2010,
-   2011 Free Software Foundation, Inc.
+   Copyright (C) 1999-2003, 2005, 2007, 2009-2012 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -208,12 +208,12 @@ hdr_date (struct header_call_args *args, void *data)
     {
       const char *p;
       struct tm tm;
-      mu_timezone tz;
+      struct mu_timezone tz;
       mu_envelope_t env;
       
       mu_message_get_envelope (args->msg, &env);
       if (mu_envelope_sget_date (env, &p) == 0
-          && mu_parse_ctime_date_time (&p, &tm, &tz) == 0)
+          && mu_scan_datetime (p, MU_DATETIME_FROM, &tm, &tz, NULL) == 0)
 	strftime (date, sizeof(date), "%a %b %e %H:%M", &tm);
     }
   return header_buf_string (args, date);
