@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-   2009, 2010 Free Software Foundation, Inc.
+   Copyright (C) 1999-2007, 2009-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 #include "mailutils/libargp.h"
@@ -23,7 +20,6 @@
 /* Global variables and constants*/
 mu_mailbox_t mbox;            /* Mailbox being operated upon */
 size_t total;                 /* Total number of messages in the mailbox */
-FILE *ofile;                  /* Output file */
 int interactive;              /* Is the session interactive */  
 
 static mu_list_t command_list;   /* List of commands to be executed after parsing
@@ -177,14 +173,14 @@ parse_opt (int key, char *arg, struct argp_state *state)
 			    _("-f requires at most one command line argument"));
 	    }
 	  else if (args->user)
-	    asprintf (&args->file, "~/%s/mbox", args->user);
+	    mu_asprintf (&args->file, "~/%s/mbox", args->user);
 	  else
 	    args->file = "~/mbox";
 	}
       else if (args->argc || (args->hint & HINT_SEND_MODE))
 	util_cache_command (&command_list, "setq mode=send");
       else if (args->user)
-	asprintf (&args->file, "%%%s", args->user);
+	mu_asprintf (&args->file, "%%%s", args->user);
       break;
       
     default:
@@ -205,7 +201,6 @@ static struct argp argp = {
 static const char *mail_capa[] = {
   "common",
   "debug",
-  "license",
   "mailbox",
   "locking",
   NULL 
@@ -224,20 +219,20 @@ mail_cmdline (void *closure, int cont MU_ARG_UNUSED)
 	{
 	  mu_mailbox_messages_count (mbox, &total);
 	  page_invalidate (0);
-	  fprintf (ofile, _("New mail has arrived.\n"));
+	  mu_printf (_("New mail has arrived.\n"));
 	}
 
       rc = ml_readline (prompt);
       
       if (ml_got_interrupt ())
 	{
-	  util_error (_("Interrupt"));
+	  mu_error (_("Interrupt"));
 	  continue;
 	}
 
       if (!rc && mailvar_get (NULL, "ignoreeof", mailvar_type_boolean, 0) == 0)
 	{
-	  util_error (_("Use \"quit\" to quit."));
+	  mu_error (_("Use \"quit\" to quit."));
 	  continue;
 	}
 
@@ -293,7 +288,7 @@ static char *default_setup[] = {
   "set recursivealiases",
   "set noinplacealiases",
   "set fromfield",
-  "set headline=\"%>%a%4m %18f %16d %3l/%-5o %s\"",
+  "set headline=\"%>%a%4m %18f %16d %3L/%-5o %s\"",
   
   /* Start in mail reading mode */
   "setq mode=read",
@@ -312,23 +307,22 @@ static char *default_setup[] = {
   "set nosendwait",
 };
 
-static int
-mail_diag_stderr_printer (void *data, mu_log_level_t level, const char *buf)
+static void
+do_and_quit (const char *command)
 {
-  if (level != MU_DIAG_ERROR)
-    fprintf (stderr, "%s: ", mu_diag_level_to_string (level));
-  fputs (buf, stderr);
-  return 0;
+  int rc = util_do_command ("%s", command);
+  mu_mailbox_close (mbox);
+  exit (rc != 0);
 }
 
 int
 main (int argc, char **argv)
 {
-  char *mode = NULL, *prompt = NULL;
+  char *mode = NULL, *prompt = NULL, *p;
   struct arguments args;
   int i, rc;
   
-  ofile = stdout;
+  mu_stdstream_setup (MU_STDSTREAM_RESET_NONE);
   set_cursor (1);
 
   /* Native Language Support */
@@ -376,13 +370,9 @@ main (int argc, char **argv)
   util_do_command ("set columns=%d", util_getcols ());
   
   /* Set the default mailer to sendmail.  */
-  {
-    char *mailer_name = alloca (strlen ("sendmail:")
-				+ strlen (PATH_SENDMAIL) + 1);
-    sprintf (mailer_name, "sendmail:%s", PATH_SENDMAIL);
-    mailvar_set ("sendmail", mailer_name, mailvar_type_string,
-		 MOPTF_OVERWRITE);
-  }
+  mailvar_set ("sendmail",
+	       xstrdup ("sendmail:" PATH_SENDMAIL), mailvar_type_string,
+	       MOPTF_OVERWRITE);
 
   args.argc = 0;
   args.argv = NULL;
@@ -394,23 +384,30 @@ main (int argc, char **argv)
 #ifdef WITH_TLS
   mu_gocs_register ("tls", mu_tls_module_init);
 #endif
-  mu_argp_init (program_version, NULL);
+  mu_argp_init (NULL, NULL);
   if (mu_app_init (&argp, mail_capa, NULL, argc, argv, 0, NULL, &args))
     exit (1);
   
   /* read system-wide mail.rc and user's .mailrc */
   if (mailvar_get (NULL, "rc", mailvar_type_boolean, 0) == 0)
     util_do_command ("source %s", SITE_MAIL_RC);
-  util_do_command ("source %s", getenv ("MAILRC"));
+  if ((p = getenv ("MAILRC")) && *p)
+    util_do_command ("source %s", getenv ("MAILRC"));
 
   util_run_cached_commands (&command_list);
 
   if (interactive)
     {
-      mu_debug_t debug;
-
-      mu_diag_get_debug (&debug);
-      mu_debug_set_print (debug, mail_diag_stderr_printer, NULL);
+      /* Reset standard error stream so that it does not print program
+	 name before the actual diagnostic message. */
+      mu_stream_t errstr;
+      int rc = mu_stdstream_strerr_create (&errstr, MU_STRERR_STDERR, 0, 0,
+					   NULL, NULL);
+      if (rc == 0)
+	{
+	  mu_stream_destroy (&mu_strerr);
+	  mu_strerr = errstr;
+	}
     }
   else
     {
@@ -445,10 +442,10 @@ main (int argc, char **argv)
       if ((rc = mu_mailbox_create_default (&mbox, args.file)) != 0)
 	{
 	  if (args.file)
-	    util_error (_("Cannot create mailbox %s: %s"), args.file,
+	    mu_error (_("Cannot create mailbox %s: %s"), args.file,
 			mu_strerror (rc));
 	  else
-	    util_error (_("Cannot create mailbox: %s"),
+	    mu_error (_("Cannot create mailbox: %s"),
 			mu_strerror (rc));
 	  exit (EXIT_FAILURE);
 	}
@@ -457,7 +454,7 @@ main (int argc, char **argv)
 	{
 	  mu_url_t url = NULL;
 	  mu_mailbox_get_url (mbox, &url);
-	  util_error (_("Cannot open mailbox %s: %s"),
+	  mu_error (_("Cannot open mailbox %s: %s"),
 		      mu_url_to_string (url), mu_strerror (rc));
 	  mu_mailbox_destroy (&mbox);
 	}
@@ -470,20 +467,23 @@ main (int argc, char **argv)
 	    {
 	      mu_url_t url = NULL;
 	      mu_mailbox_get_url (mbox, &url);
-	      util_error (_("Cannot read mailbox %s: %s"),
+	      mu_error (_("Cannot read mailbox %s: %s"),
 			  mu_url_to_string (url), mu_strerror (rc));
 	      exit (EXIT_FAILURE);
 	    }
 
 	  if (strcmp (mode, "exist") == 0)
-	    return (total < 1) ? 1 : 0;
+	    {
+	      mu_mailbox_close (mbox);
+	      return (total < 1) ? 1 : 0;
+	    }
 	  else if (strcmp (mode, "print") == 0)
-	    return util_do_command ("print *");
+	    do_and_quit ("print *");
 	  else if (strcmp (mode, "headers") == 0)
-	    return util_do_command ("from *");
+	    do_and_quit ("from *");
 	  else if (strcmp (mode, "read"))
 	    {
-	      util_error (_("Unknown mode `%s'"), mode);
+	      mu_error (_("Unknown mode `%s'"), mode);
 	      util_do_command ("quit");
 	      return 1;
 	    }
@@ -494,10 +494,10 @@ main (int argc, char **argv)
 	      || mailvar_get (NULL, "emptystart", mailvar_type_boolean, 0)))
         {
 	  if (args.file)
-	    fprintf (ofile, _("%s: 0 messages\n"), args.file);
+	    mu_printf (_("%s: 0 messages\n"), args.file);
 	  else
-	    fprintf (ofile, _("No mail for %s\n"),
-		     args.user ? args.user : mail_whoami ());
+	    mu_printf (_("No mail for %s\n"),
+			      args.user ? args.user : mail_whoami ());
           return 1;
         }
 
@@ -510,7 +510,7 @@ main (int argc, char **argv)
 
       mailvar_get (&prompt, "prompt", mailvar_type_string, 0);
       mail_mainloop (mail_cmdline, (void*) prompt, 1);
-      fprintf (ofile, "\n");
+      mu_printf ("\n");
       util_do_command ("quit");
       return 0;
     }
@@ -560,11 +560,10 @@ mail_mainloop (char *(*input) (void *, int),
 int
 mail_warranty (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED)
 {
-  fputs (_("GNU Mailutils -- a suite of utilities for electronic mail\n"
-           "Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,\n"
-           "2007, 2009 Free Software Foundation, Inc.\n\n"),
-           ofile);
-  fputs (
+  mu_printf (
+	 _("GNU Mailutils -- a suite of utilities for electronic mail\n"
+           "Copyright (C) 1999-2011 Free Software Foundation, Inc.\n\n"));
+  mu_printf (
   _("   GNU Mailutils is free software; you can redistribute it and/or modify\n"
     "   it under the terms of the GNU General Public License as published by\n"
     "   the Free Software Foundation; either version 3 of the License, or\n"
@@ -576,12 +575,10 @@ mail_warranty (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED)
     "   GNU General Public License for more details.\n"
     "\n"
     "   You should have received a copy of the GNU General Public License along\n"
-    "   with GNU Mailutils; if not, write to the Free Software Foundation,\n"
-    "   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA\n"
+    "   with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>.\n" 
     "\n"
     "\n"
-),
-    ofile);
+));
 
   return 0;
 }
