@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-   2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 1999-2007, 2009-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -475,7 +474,7 @@ util_get_homedir ()
 char *
 util_fullpath (const char *inpath)
 {
-  return mu_tilde_expansion (inpath, "/", NULL);
+  return mu_tilde_expansion (inpath, MU_HIERARCHY_DELIMITER, NULL);
 }
 
 char *
@@ -861,17 +860,13 @@ util_merge_addresses (char **addr_str, const char *value)
   rc = mu_address_union (&addr, new_addr);
   if (rc == 0)
     {
-      size_t n;
-
-      rc = mu_address_to_string (addr, NULL, 0, &n);
+      char *val;
+      
+      rc = mu_address_aget_printable (addr, &val);
       if (rc == 0)
 	{
 	  free (*addr_str);
-	  *addr_str = malloc (n + 1);
-	  if (!*addr_str)
-	    rc = ENOMEM;
-	  else
-	    mu_address_to_string (addr, *addr_str, n + 1, &n);
+	  *addr_str = val;
 	}
     }
 
@@ -972,15 +967,12 @@ util_header_expand (mu_header_t *phdr)
 	  
 	  if (addr)
 	    {
-	      char *newvalue;
-	      size_t n = 0;
+	      const char *newvalue;
 	      
-	      mu_address_to_string (addr, NULL, 0, &n);
-	      newvalue = xmalloc (n + 1);
-	      mu_address_to_string (addr, newvalue, n + 1, NULL);
+	      rc = mu_address_sget_printable (addr, &newvalue);
+	      if (rc == 0)
+		mu_header_set_value (hdr, name, newvalue, 1);
 	      mu_address_destroy (&addr);
-	      mu_header_set_value (hdr, name, newvalue, 1);
-	      free (newvalue);
 	    }
 	}
       else
@@ -1061,7 +1053,7 @@ _run_and_free (void *item, void *data)
 void
 util_run_cached_commands (mu_list_t *list)
 {
-  mu_list_do (*list, _run_and_free, NULL);
+  mu_list_foreach (*list, _run_and_free, NULL);
   mu_list_destroy (list);
 }
 
