@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2008,
-   2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 1999-2003, 2005-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,10 +27,6 @@
 
 #include <dirent.h>
 
-#define obstack_chunk_alloc malloc
-#define obstack_chunk_free free
-#include <obstack.h>
-
 static char doc[] = N_("GNU MH folder")"\v"
 N_("Use -help to obtain the list of traditional MH options.");
 static char args_doc[] = N_("[ACTION] [MSG]");
@@ -243,10 +238,7 @@ struct folder_info
   size_t others;           /* Number of non-message files */ 
 };
 
-struct obstack folder_info_stack; /* Memory storage for folder infp */
-struct folder_info *folder_info;  /* Finalized array of information
-				     structures */
-size_t folder_info_count;         /* Number of the entries in the array */
+mu_list_t folder_info_list; /* Memory storage for folder info */
 
 size_t message_count;             /* Total number of messages */
 
@@ -255,9 +247,10 @@ int name_prefix_len;              /* Length of the mu_path_folder_dir */
 void
 install_folder_info (const char *name, struct folder_info *info)
 {
-  info->name = strdup (name) + name_prefix_len;
-  obstack_grow (&folder_info_stack, info, sizeof (*info));
-  folder_info_count++;
+  struct folder_info *new_info = xmalloc (sizeof (*new_info));
+  *new_info = *info;
+  new_info->name = strdup (new_info->name + name_prefix_len);
+  mu_list_append (folder_info_list, new_info);
   message_count += info->message_count;
 }
 
@@ -321,7 +314,7 @@ _scan (const char *name, size_t depth)
       if (fast_mode && depth > 0)
 	{
 	  memset (&info, 0, sizeof (info));
-	  info.name = strdup (name);
+	  info.name = name;
 	  install_folder_info (name, &info);
 	  closedir (dir);
 	  return;
@@ -382,60 +375,69 @@ _scan (const char *name, size_t depth)
   if (depth > 0)
     install_folder_info (name, &info);
 }
-    
+
+static int
+_folder_info_printer (void *item, void *data)
+{
+  struct folder_info *info = item;
+  int len = strlen (info->name);
+
+  if (len < 22)
+    printf ("%22.22s", info->name);
+  else
+    printf ("%s", info->name);
+  
+  if (strcmp (info->name, mh_current_folder ()) == 0)
+    printf ("+");
+  else
+    printf (" ");
+  
+  if (info->message_count)
+    {
+      printf (ngettext(" has %4lu message  (%4lu-%4lu)",
+		       " has %4lu messages (%4lu-%4lu)",
+		       info->message_count),
+	      (unsigned long) info->message_count,
+	      (unsigned long) info->min,
+	      (unsigned long) info->max);
+      if (info->cur)
+	printf ("; cur=%4lu", (unsigned long) info->cur);
+    }
+  else
+    {
+      printf (_(" has no messages"));
+    }
+  
+  if (info->others)
+    {
+      if (!info->cur)
+	printf (";           ");
+      else
+	printf ("; ");
+      printf (_("(others)"));
+    }
+  printf (".\n");
+  return 0;
+}
+
+static int
+_folder_name_printer (void *item, void *data)
+{
+  struct folder_info *info = item;
+  printf ("%s\n", info->name);
+  return 0;
+}
+
 static void
 print_all ()
 {
-  struct folder_info *info, *end = folder_info + folder_info_count;
-
-  for (info = folder_info; info < end; info++)
-    {
-      int len = strlen (info->name);
-      if (len < 22)
-	printf ("%22.22s", info->name);
-      else
-	printf ("%s", info->name);
-      
-      if (strcmp (info->name, mh_current_folder ()) == 0)
-	printf ("+");
-      else
-	printf (" ");
-      
-      if (info->message_count)
-	{
-	  printf (ngettext(" has %4lu message  (%4lu-%4lu)",
-			   " has %4lu messages (%4lu-%4lu)",
-			   info->message_count),
-		  (unsigned long) info->message_count,
-		  (unsigned long) info->min,
-		  (unsigned long) info->max);
-	  if (info->cur)
-	    printf ("; cur=%4lu", (unsigned long) info->cur);
-	}
-      else
-	{
-	  printf (_(" has no messages"));
-	}
-      
-      if (info->others)
-	{
-	  if (!info->cur)
-	    printf (";           ");
-	  else
-	    printf ("; ");
-	  printf (_("(others)"));
-	}
-      printf (".\n");
-    }
+  mu_list_foreach (folder_info_list, _folder_info_printer, NULL);
 }
 
 static void
 print_fast ()
 {
-  struct folder_info *info, *end = folder_info + folder_info_count;
-
-  for (info = folder_info; info < end; info++)
-    printf ("%s\n", info->name);
+  mu_list_foreach (folder_info_list, _folder_name_printer, NULL);
 }
 
 static int
@@ -449,7 +451,7 @@ action_print ()
     name_prefix_len++;
   name_prefix_len++;  /* skip past the slash */
 
-  obstack_init (&folder_info_stack);
+  mu_list_create (&folder_info_list);
 
   if (show_all)
     {
@@ -462,9 +464,7 @@ action_print ()
       free (p);
     }
   
-  folder_info = obstack_finish (&folder_info_stack);
-  qsort (folder_info, folder_info_count, sizeof (folder_info[0]),
-	 folder_info_comp);
+  mu_list_sort (folder_info_list, folder_info_comp);
 
   if (fast_mode)
     print_fast ();
@@ -477,13 +477,16 @@ action_print ()
 
       if (OPTION_IS_SET (print_total))
 	{
+	  size_t folder_count;
+
+	  mu_list_count (folder_info_list, &folder_count);
 	  printf ("\n%24.24s=", _("TOTAL"));
 	  printf (ngettext ("%4lu message  ", "%4lu messages ",
 			    message_count),
 		  (unsigned long) message_count);
 	  printf (ngettext ("in %4lu folder", "in %4lu folders",
-			    folder_info_count),
-		  (unsigned long) folder_info_count);
+			    folder_count),
+		  (unsigned long) folder_count);
 	  printf ("\n");
 	}
     }
@@ -769,9 +772,10 @@ pack_xlate (struct pack_tab *pack_tab, size_t count, size_t n)
 static int
 _fixup (const char *name, const char *value, struct fixup_data *fd, int flags)
 {
-  size_t i, j;
+  size_t i;
+  int rc;
   struct mu_wordsplit ws;
-  mh_msgset_t msgset;
+  mu_msgset_t msgset;
 
   if (verbose)
     fprintf (stderr, "Sequence `%s'...\n", name);
@@ -783,20 +787,32 @@ _fixup (const char *name, const char *value, struct fixup_data *fd, int flags)
       return 0;
     }
 
-  msgset.list = xcalloc (ws.ws_wordc, sizeof msgset.list[0]);
-  for (i = j = 0; i < ws.ws_wordc; i++)
+  rc = mu_msgset_create (&msgset, NULL, MU_MSGSET_UID);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_msgset_create", NULL, rc);
+      exit (1);
+    }
+  
+  for (i = 0; i < ws.ws_wordc; i++)
     {
       size_t n = pack_xlate (fd->pack_tab, fd->count,
 			     strtoul (ws.ws_wordv[i], NULL, 0));
       if (n)
-	msgset.list[j++] = n;
+	{
+	  rc = mu_msgset_add_range (msgset, n, n, MU_MSGSET_UID);
+	  if (rc)
+	    {
+	      mu_diag_funcall (MU_DIAG_ERROR, "mu_msgset_add_range", NULL, rc);
+	      exit (1);
+	    }
+	}
     }
-  msgset.count = j;
 
   mu_wordsplit_free (&ws);
   
-  mh_seq_add (fd->mbox, name, &msgset, flags | SEQ_ZERO);
-  free (msgset.list);
+  mh_seq_add (fd->mbox, name, msgset, flags | SEQ_ZERO);
+  mu_msgset_free (msgset);
 
   if (verbose)
     {
@@ -940,7 +956,7 @@ int
 main (int argc, char **argv)
 {
   int index = 0;
-  mh_msgset_t msgset;
+  mu_msgset_t msgset;
 
   /* Native Language Support */
   MU_APP_INIT_NLS ();
@@ -967,9 +983,11 @@ main (int argc, char **argv)
     
   if (argc - index == 1)
     {
-      mu_mailbox_t mbox = mh_open_folder (mh_current_folder (), MU_STREAM_RDWR);
-      mh_msgset_parse (mbox, &msgset, argc - index, argv + index, "cur");
-      mh_msgset_current (mbox, &msgset, 0);
+      mu_mailbox_t mbox = mh_open_folder (mh_current_folder (),
+					  MU_STREAM_RDWR);
+      mh_msgset_parse (&msgset, mbox, argc - index, argv + index, "cur");
+      mh_msgset_first_current (mbox, msgset);
+      mu_msgset_free (msgset);
       mh_global_save_state ();
       mu_mailbox_close (mbox);
       mu_mailbox_destroy (&mbox);
