@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2007, 2010, 2011 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2001, 2007, 2010-2012 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -94,7 +94,7 @@ mh_read_profile ()
   
   p = getenv ("MH");
   if (p)
-    p = mu_tilde_expansion (p, "/", NULL);
+    p = mu_tilde_expansion (p, MU_HIERARCHY_DELIMITER, NULL);
   else
     {
       char *home = mu_get_homedir ();
