@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005, 2006, 2007, 2008,
-   2009 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* MH send command */
 
@@ -25,11 +22,10 @@
 #include <stdarg.h>
 #include <pwd.h>
 
-const char *program_version = "send (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU MH send")"\v"
 N_("Options marked with `*' are not yet implemented.\n\
 Use -help to obtain the list of traditional MH options.");
-static char args_doc[] = N_("file [file...]");
+static char args_doc[] = N_("FILE [FILE...]");
 
 /* GNU options */
 static struct argp_option options[] = {
@@ -77,36 +73,35 @@ static struct argp_option options[] = {
   {"nowatch",       ARG_NOWATCH,       NULL, OPTION_HIDDEN, "" },
   {"width",         ARG_WIDTH,         N_("NUMBER"), 0,
    N_("* make header fields no longer than NUMBER columns") },
-  {"license", ARG_LICENSE, 0,      0,
-   N_("display software license"), -1},
   { 0 }
 };
 
 /* Traditional MH options */
 struct mh_option mh_option[] = {
-  {"alias",         1, 0, "aliasfile" },
-  {"draft",         5, 0, NULL },
-  {"draftfolder",   6, 0, "folder" },
-  {"draftmessage",  6, 0, "message"},
-  {"nodraftfolder", 3, 0, NULL },
-  {"filter",        2, 0, "filterfile"},
-  {"nofilter",      3, 0, NULL },
-  {"format",        4, MH_OPT_BOOL, NULL},
-  {"forward",       4, MH_OPT_BOOL, NULL},
-  {"mime",          2, MH_OPT_BOOL, NULL},
-  {"msgid",         2, MH_OPT_BOOL, NULL},
-  {"push",          2, MH_OPT_BOOL, NULL},
-  {"preserve",      2, MH_OPT_BOOL, NULL},
-  {"keep",          1, MH_OPT_BOOL, NULL},
-  {"split",         1, 0, "seconds"},
-  {"verbose",       1, MH_OPT_BOOL, NULL},
-  {"watch",         2, MH_OPT_BOOL, NULL},
-  {"width",         2, 0, NULL },
-  { 0 }
+  { "alias",         MH_OPT_ARG, "aliasfile" },
+  { "draft" },
+  { "draftfolder",   MH_OPT_ARG, "folder" },
+  { "draftmessage",  MH_OPT_ARG, "message" },
+  { "nodraftfolder" },
+  { "filter",        MH_OPT_ARG, "filterfile" },
+  { "nofilter" },
+  { "format",        MH_OPT_BOOL },
+  { "forward",       MH_OPT_BOOL },
+  { "mime",          MH_OPT_BOOL },
+  { "msgid",         MH_OPT_BOOL },
+  { "push",          MH_OPT_BOOL },
+  { "preserve",      MH_OPT_BOOL },
+  { "keep",          MH_OPT_BOOL },
+  { "split",         MH_OPT_ARG, "seconds" },
+  { "verbose",       MH_OPT_BOOL },
+  { "watch",         MH_OPT_BOOL },
+  { "width" },
+  { NULL }
 };
 
 static int use_draft;            /* Use the prepared draft */
-static const char *draft_folder; /* Use this draft folder */
+static const char *draftfolder;  /* Use this draft folder */
+static char *draftmessage = "cur";
 static int reformat_recipients;  /* --format option */
 static int forward_notice;       /* Forward the failure notice to the sender,
 				    --forward flag */
@@ -131,8 +126,8 @@ static int keep_files;           /* Keep draft files */
     watch_printf c;\
 } while (0)
 
-static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+static error_t
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   char *p;
   
@@ -156,44 +151,52 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
 	
     case ARG_DRAFTFOLDER:
-      draft_folder = arg;
+      draftfolder = arg;
       break;
       
     case ARG_NODRAFTFOLDER:
-      draft_folder = NULL;
+      draftfolder = NULL;
       break;
       
     case ARG_DRAFTMESSAGE:
-      if (!draft_folder)
-	draft_folder = mh_global_profile_get ("Draft-Folder",
-					      mu_folder_directory ());
+      draftmessage = arg;
       break;
       
     case ARG_FILTER:
+      mh_opt_notimpl ("-filter");
+      break;
+
     case ARG_NOFILTER:
-      return 1;
-      
+      mh_opt_notimpl ("-nofilter");
+      break;
+ 
     case ARG_FORMAT:
+      mh_opt_notimpl_warning ("-format"); 
       reformat_recipients = is_true (arg);
       break;
       
     case ARG_NOFORMAT:
+      mh_opt_notimpl_warning ("-noformat"); 
       reformat_recipients = 0;
       break;
       
     case ARG_FORWARD:
+      mh_opt_notimpl_warning ("-forward");
       forward_notice = is_true (arg);
       break;
       
     case ARG_NOFORWARD:
+      mh_opt_notimpl_warning ("-noforward");
       forward_notice = 0;
       break;
       
     case ARG_MIME:
+      mh_opt_notimpl_warning ("-mime");
       mime_encaps = is_true (arg);
       break;
       
     case ARG_NOMIME:
+      mh_opt_notimpl_warning ("-nomime");
       mime_encaps = 0;
       break;
       
@@ -244,6 +247,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_WIDTH:
+      mh_opt_notimpl_warning ("-width");
       width = strtoul (arg, &p, 10);
       if (*p)
 	{
@@ -252,12 +256,8 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
 	}
       break;
       
-    case ARG_LICENSE:
-      mh_license (argp_program_version);
-      break;
-
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
@@ -281,14 +281,14 @@ struct list_elt           /* Element of the send list */
 };
 
 static mu_list_t mesg_list;
-static mh_context_t *mts_profile;
+static mu_property_t mts_profile;
 
 int
 check_file (char *name)
 {
   struct list_elt *elt;
   mu_message_t msg;
-  char *file_name = mh_expand_name (draft_folder, name, 0);
+  char *file_name = mh_expand_name (draftfolder, name, 0);
   
   msg = mh_file_to_message (NULL, file_name);
   if (!msg)
@@ -311,36 +311,49 @@ check_file (char *name)
 void
 read_mts_profile ()
 {
+  char *name;
   const char *p;
   char *hostname = NULL;
   int rc;
-  mh_context_t *local_profile;
 
-  p = mh_expand_name (MHLIBDIR, "mtstailor", 0);
-  mts_profile = mh_context_create (p, 1);
-  mh_context_read (mts_profile);
-
-  p = mu_tilde_expansion ("~/.mtstailor", "/", NULL);
-  local_profile = mh_context_create (p, 1);
-  if (mh_context_read (local_profile) == 0)
-    mh_context_merge (mts_profile, local_profile);
-  mh_context_destroy (&local_profile);
-
-  if ((p = mh_context_get_value (mts_profile, "localname", NULL)))
+  name = getenv ("MTSTAILOR");
+  if (name)
+    mts_profile = mh_read_property_file (name, 1);
+  else
     {
-      hostname = xstrdup (p);
-      mu_set_user_email_domain (p);
-    }
-  else if ((rc = mu_get_host_name (&hostname)))
-    mu_error (_("cannot get system host name: %s"), mu_strerror (rc));
+      mu_property_t local_profile;
 
-  if ((p = mh_context_get_value (mts_profile, "localdomain", NULL)))
+      name = mh_expand_name (MHLIBDIR, "mtstailor", 0);
+      mts_profile = mh_read_property_file (name, 1);
+
+      name = mu_tilde_expansion ("~/.mtstailor", MU_HIERARCHY_DELIMITER, NULL);
+      local_profile = mh_read_property_file (name, 1);
+
+      mh_property_merge (mts_profile, local_profile);
+      mu_property_destroy (&local_profile);
+    }
+
+  rc = mu_property_aget_value (mts_profile, "localname", &hostname);
+  if (rc == MU_ERR_NOENT)
+    {
+      rc = mu_get_host_name (&hostname);
+      if (rc)
+	{
+	  mu_error (_("cannot get system host name: %s"), mu_strerror (rc));
+	  exit (1);
+	}
+    }
+  else if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_profile_aget_value", "localname", rc);
+      exit (1);
+    }
+
+  rc = mu_property_sget_value (mts_profile, "localdomain", &p);
+  if (rc == 0)
     {
       char *newdomain;
 
-      if (!hostname)
-	exit (1);
-      
       newdomain = xmalloc (strlen (hostname) + 1 + strlen (p) + 1);
       strcpy (newdomain, hostname);
       strcat (newdomain, ".");
@@ -353,8 +366,15 @@ read_mts_profile ()
 	  exit (1);
 	}
     }
-
-  if ((p = mh_context_get_value (mts_profile, "username", NULL)))
+  else if (rc != MU_ERR_NOENT)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_profile_sget_value",
+		       "localdomain", rc);
+      exit (1);
+    }
+  
+  rc = mu_property_sget_value (mts_profile, "username", &p);
+  if (rc == 0)
     {
       size_t len;
       const char *domain;
@@ -383,15 +403,21 @@ read_mts_profile ()
       
       free (newemail);
     }
+  else if (rc != MU_ERR_NOENT)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_profile_sget_value",
+		       "username", rc);
+      exit (1);
+    }
 }
 
 
 mu_mailer_t
 open_mailer ()
 {
-  const char *url = mh_context_get_value (mts_profile,
-					  "url",
-					  "sendmail:/usr/sbin/sendmail");
+  const char *url = mu_mhprop_get_value (mts_profile,
+					 "url",
+					 "sendmail:/usr/sbin/sendmail");
   mu_mailer_t mailer;
   int status;
     
@@ -405,9 +431,8 @@ open_mailer ()
 
   if (verbose)
     {
-      mu_debug_t debug = NULL;
-      mu_mailer_get_debug (mailer, &debug);
-      mu_debug_set_level (debug, MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
+      mu_debug_set_category_level (MU_DEBCAT_MAILER,
+                                   MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
     }
 
   WATCH ((_("Opening mailer %s"), url));
@@ -449,11 +474,10 @@ get_sender_personal ()
 static void
 set_address_header (mu_header_t hdr, char *name, mu_address_t addr)
 {
-  size_t s = mu_address_format_string (addr, NULL, 0);
-  char *value = xmalloc (s + 1);
-  mu_address_format_string (addr, value, s);
-  mu_header_set_value (hdr, name, value, 1);
-  free (value);
+  const char *value;
+  if (mu_address_sget_printable (addr, &value) == 0)
+    mu_header_set_value (hdr, name, value, 1);
+  /* FIXME: Error reporting */
 }
 
 void
@@ -495,49 +519,53 @@ fix_fcc (mu_message_t msg)
   mu_message_get_header (msg, &hdr);
   if (mu_header_aget_value (hdr, MU_HEADER_FCC, &fcc) == 0)
     {
-      int i, argc;
-      char **argv;
+      struct mu_wordsplit ws;
       int need_fixup = 0;
       size_t fixup_len = 0;
-      
-      mu_argcv_get (fcc, ",", NULL, &argc, &argv);
-      for (i = 0; i < argc; i += 2)
+
+      ws.ws_delim = ",";
+      if (mu_wordsplit (fcc, &ws,
+			MU_WRDSF_DEFFLAGS | MU_WRDSF_DELIM | MU_WRDSF_WS))
 	{
-	  if (strchr ("+%~/=", argv[i][0]) == NULL)
-	    {
-	      need_fixup++;
-	      fixup_len ++;
-	    }
-	  fixup_len += strlen (argv[i]);
+	  mu_error (_("cannot split line `%s': %s"), fcc,
+		    mu_wordsplit_strerror (&ws));
 	}
-
-      if (need_fixup)
+      else
 	{
-	  char *p;
-
-	  /* the new fcc string contains: folder names - fixup_len characters
-	     long, (argc - 2)/2 comma-space pairs and a terminating
-	     nul */
-	  fcc = realloc (fcc, fixup_len + argc - 2 + 1);
-	  for (i = 0, p = fcc; i < argc; i++)
+	  size_t i;
+	  
+	  for (i = 0; i < ws.ws_wordc; i += 2)
 	    {
-	      if (i % 2 == 0)
+	      if (strchr ("+%~/=", ws.ws_wordv[i][0]) == NULL)
 		{
-		  if (strchr ("+%~/=", argv[i][0]) == NULL)
-		    *p++ = '+';
-		  strcpy (p, argv[i]);
-		  p += strlen (argv[i]);
+		  need_fixup++;
+		  fixup_len ++;
 		}
-	      else
+	      fixup_len += strlen (ws.ws_wordv[i]);
+	    }
+
+	  if (need_fixup)
+	    {
+	      char *p;
+	      
+	      /* the new fcc string contains: folder names - fixup_len
+		 characters, ws.ws_wordc - 1 comma-space pairs and a
+		 terminating nul */
+	      fcc = realloc (fcc, fixup_len + ws.ws_wordc - 1 + 1);
+	      for (i = 0, p = fcc; i < ws.ws_wordc; i++)
 		{
+		  if (strchr ("+%~/=", ws.ws_wordv[i][0]) == NULL)
+		    *p++ = '+';
+		  strcpy (p, ws.ws_wordv[i]);
+		  p += strlen (p);
 		  *p++ = ',';
 		  *p++ = ' ';
 		}
+	      *p = 0;
 	    }
-	  *p = 0;
 	}
 
-      mu_argcv_free (argc, argv);
+      mu_wordsplit_free (&ws);
 
       if (need_fixup)
 	{
@@ -645,7 +673,7 @@ _action_send (void *item, void *data)
 	  const char *pers = get_sender_personal ();
 	  if (pers)
 	    {
-	      asprintf (&from, "\"%s\" <%s>", pers, email);
+	      mu_asprintf (&from, "\"%s\" <%s>", pers, email);
 	      free (email);
 	    }
 	  else
@@ -661,8 +689,8 @@ _action_send (void *item, void *data)
 
       if (mu_header_get_value (hdr, MU_HEADER_X_MAILER, NULL, 0, &n))
 	{
-	  const char *p = mh_context_get_value (mts_profile,
-						"x-mailer", "yes");
+	  const char *p = mu_mhprop_get_value (mts_profile,
+					       "x-mailer", "yes");
 
 	  if (!strcmp (p, "yes"))
 	    mu_header_set_value (hdr, MU_HEADER_X_MAILER,
@@ -709,15 +737,91 @@ _action_send (void *item, void *data)
 }
 
 static int
-do_send (int argc, char **argv)
+_add_to_mesg_list (size_t num, mu_message_t msg, void *data)
 {
-  int i, rc;
-  char *p;
+  char const *path = data;
+  struct list_elt *elt;
+  size_t uid;
+  int rc;
   
-  /* Verify all arguments */
-  for (i = 0; i < argc; i++)
-    if (check_file (argv[i]))
-      return 1;
+  elt = xmalloc (sizeof *elt);
+  elt->msg = msg;
+  mu_message_get_uid (msg, &uid);
+  elt->file_name = mu_make_file_name (path, mu_umaxtostr (0, uid));
+  rc = mu_list_append (mesg_list, elt);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_list_append", NULL, rc);
+      exit (1);
+    }
+  return 0;
+}
+
+int
+main (int argc, char **argv)
+{
+  mu_mailbox_t mbox = NULL;
+  int index;
+  char *p;
+  int rc;
+  
+  MU_APP_INIT_NLS ();
+  
+  mh_argp_init ();
+  mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
+		 opt_handler, NULL, &index);
+
+  mh_read_aliases ();
+  
+  argc -= index;
+  argv += index;
+
+  if (draftfolder)
+    {
+      mu_msgset_t msgset;
+      mu_url_t url;
+      const char *path;
+      
+      mbox = mh_open_folder (draftfolder, MU_STREAM_RDWR|MU_STREAM_CREAT);
+      mh_msgset_parse (&msgset, mbox, argc, argv, draftmessage);
+      mu_mailbox_get_url (mbox, &url);
+      mu_url_sget_path (url, &path);
+      if ((rc = mu_list_create (&mesg_list)))
+	{
+	  mu_error (_("cannot create message list: %s"), mu_strerror (rc));
+	  exit (1);
+	}
+      mu_msgset_foreach_message (msgset, _add_to_mesg_list, (void*)path);
+      
+      mu_msgset_free (msgset);
+    }
+  else
+    {
+      int i;
+      
+      if (argc == 0)
+	{
+	  char *xargv[2];
+	  struct stat st;
+	  
+	  xargv[0] = mh_draft_name ();
+
+	  if (stat (xargv[0], &st))
+	    {
+	      mu_diag_funcall (MU_DIAG_ERROR, "stat", xargv[0], errno);
+	      return 1;
+	    }
+
+	  if (!use_draft && !mh_usedraft (xargv[0]))
+	    exit (0);
+	  xargv[1] = NULL;
+	  argv = xargv;
+	  argc = 1;
+	}
+      for (i = 0; i < argc; i++)
+	if (check_file (argv[i]))
+	  return 1;
+    }
 
   /* Process the mtstailor file and detach from the console if
      required */
@@ -731,49 +835,14 @@ do_send (int argc, char **argv)
 
   /* Prepend url specifier to the folder dir. We won't need this
      when the default format becomes configurable */
-  asprintf (&p, "mh:%s", mu_folder_directory ());
+  mu_asprintf (&p, "mh:%s", mu_folder_directory ());
   mu_set_folder_directory (p);
   free (p);
   
   /* Finally, do the work */
-  rc = mu_list_do (mesg_list, _action_send, NULL);
-  return rc;
+  rc = mu_list_foreach (mesg_list, _action_send, NULL);
+
+  mu_mailbox_destroy (&mbox);
+  return !!rc;
 }
-	  
-int
-main (int argc, char **argv)
-{
-  int index;
-  
-  MU_APP_INIT_NLS ();
-  
-  mh_argp_init (program_version);
-  mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
-		 opt_handler, NULL, &index);
 
-  mh_read_aliases ();
-  
-  argc -= index;
-  argv += index;
-
-  if (argc == 0)
-    {
-      struct stat st;
-      static char *xargv[2];
-      xargv[0] = mh_draft_name ();
-
-      if (stat (xargv[0], &st))
-	{
-	  mu_diag_funcall (MU_DIAG_ERROR, "stat", xargv[0], errno);
-	  return 1;
-	}
-
-      if (!use_draft && !mh_usedraft (xargv[0]))
-	exit (0);
-      xargv[1] = NULL;
-      argv = xargv;
-      argc = 1;
-    }
-
-  return do_send (argc, argv);  
-}
